Set-StrictMode -Version Latest
#updated 20230731
#oagEmulationRomCodes.ps1

function Get-RegionFromTitleIdSony([string]$TitleId)
{
#updated 20230728
# most of this sourced from PS3 Developer wiki
# WARNING: the PCS codes for Vita are not confirmed
	[string]$sResult = ''

	if ($TitleId.length -ne 9)
	{
		return
	}

	if ($TitleId.StartsWith('NP'))
	{
		$sResult = switch ($TitleId[2])
		{
			'A' {'Asia'}
			'E' {'Europe'}
			'H' {'Hong Kong'}
			'J' {'Japan'}
			'K' {'Korea'}
			'U' {'USA'}
			# I Internal?
			# X Firmware
		}
	} elseif ($TitleId.StartsWith('PCS')) {
# VITA CODES REVERSE ENGINEERED
		$sResult = switch ($TitleId[3])
		{
			'A' {'USA'}
			'B' {'Europe'}
			'C' {'Japan'}
			'D' {'Asia'}
			'E' {'USA'}
			'F' {'Europe'}
			'G' {'Japan'}
			'H' {'Asia'}
		}
	} else {
		$sResult = switch ($TitleId[2])
		{
			'A' {'Asia'}
			'C' {'China'} # not apply to digital games?
			'E' {'Europe'}
			'H' {'Hong Kong'}
			'J' {'Japan'}
			'K' {'Korea'}
			'P' {'Japan'} # not apply to digital games?
			'U' {'USA'}
		}
	}
	return $sResult
}

function Get-RegionFromNintendoGameID([string]$GameID)
{
# works for GameCube, Wii and Wii U
	[string]$sResult = ''
	[int]$iPos = 0

	$iPos = switch ($($GameID.length))
	{
		10 {9} # Nintendo ProductCode ?
		6 {3} # Nintendo GameID6
		4 {3} # Nintendo GameID4
	}

	if ($iPos -gt 0)
	{
		$sResult = switch ($($GameID[$iPos]))
		{
			'D' {'Germany'}
			'E' {'USA'}
			'F' {'France'}
			'I' {'Italy'}
			'J' {'Japan'}
			'K' {'Korea'}
			'P' {'Europe'} # all PAL territories unless a game has a country release
			'R' {'Russia'}
			'S' {'Spain'}
			'W' {'Taiwan'}
			'Z' {'Scandinavia'}
		}
	}
	return $sResult
}

function Get-RegionFromXboxRegionID($ID)
{
# WARNING: regions are guessed because of lack of information

# NOTES: appears to be using binary
# 001 = USA (North America?)
# 010 = Japan
# 011 = US + Japan = USA-JPN (NTSC?)
# 100 = Europe? PAL?
# 111 = all regions = World
	switch ($ID)
	{
		1 {'USA'}
		2 {'Japan'}
		3 {'Japan, USA'}
		4 {'Europe'}
		7 {'World'}
	}
}

function Get-RegionNameFromWiiURegionId([string]$sID)
{
#TODO MED finish adding codes
	[string]$sResult = ''
	$sResult = switch ($sID)
	{
		'2' {'USA'}
		'4' {'Europe'}
	}
	return $sResult
}