Set-StrictMode -Version Latest
#updated 20230810
#oagGamePath.ps1

[byte]$miVerPN = 8

function Get-PlayniteNamePCWin
{
	switch ($miVerPN)
	{
		8 {return 'PC'}
		10 {return 'PC (Windows)'}
	}
}

function Get-PlayniteNamePCDOS
{
	switch ($miVerPN)
	{
		8 {return 'DOS'}
		10 {return 'PC (DOS)'}
	}
}

function Get-ActualInstallationPathForGame(
	[parameter(mandatory)][object]$game,
	[switch]$Verify,
	[switch]$Silent
) {
#updated 20230810
#required function: Get-EmuLibraryRomFullName, Get-WindowsFriendlyNameFolder
	[string]$sResult = ''
	[bool]$bRemovedSubFolderAny = $false
	[bool]$bRemovedSubFolderThisLoop = $false
	[bool]$bWalkBackPath = $true
	[bool]$bIsEmuLibrary = $false
	[bool]$bIsConsole = $true
	[string]$sCurrentFolder = ''
	[string]$sFriendlyGameName1 = ''
	[string]$sFriendlyGameName2 = ''
	[string]$sGameInstallDir = ''
	[string]$sFolderSubstring = ''

	[string]$sNamePCWin = Get-PlayniteNamePCWin
	[string]$sNamePCDOS = Get-PlayniteNamePCDOS



	# test if EmuLibrary (should use plugin Id)
	if (($game.Id) -match '\|')
	{
		$bIsEmuLibrary = $true
	}

	if ($bIsEmuLibrary)
	{
		$sGameInstallDir = Get-EmuLibraryRomFullName -game $game
		if ($sGameInstallDir)
		{
			$sGameInstallDir = Split-Path -Path $sGameInstallDir -parent
		}
	} else {
		if ($($game.InstallDirectory))
		{
			$sGameInstallDir = $PlayniteApi.ExpandGameVariables($game, $game.InstallDirectory)
		} else {
			$sMsg = "no InstallDirectory recorded in Playnite for `"$($game.name)`" ($($game.Platform.name))"
			$__logger.Error($sMsg)
			if (!$Silent)
			{
				$null = $PlayniteApi.Dialogs.ShowMessage($sMsg,'ERROR')
			}
		}
	}

	if ($sGameInstallDir)
	{
		$sResult = $sGameInstallDir.TrimEnd('\') + '\'
	} else {
		return $sResult
	}

	# check path exists - 1st check
	if ($Verify)
	{
		if (![System.IO.Directory]::Exists($sResult))
		{
			$sMsg = "Game install directory does not exist.`r`n`r`n'$sResult'"
			$__logger.Error($sMsg)
			if (!$Silent)
			{
				$null = $PlayniteApi.Dialogs.ShowMessage($sMsg,'ERROR')
			}
			$sResult = ''
			return $sResult
		}
	}


	$sFriendlyGameName1 = "$($game.name)"
	$sFriendlyGameName2 = $sFriendlyGameName1.replace(': ',' - ')

	$sFriendlyGameName1 = Get-WindowsFriendlyNameFolder -value $sFriendlyGameName1
	$sFriendlyGameName2 = Get-WindowsFriendlyNameFolder -value $sFriendlyGameName2


#TODO LOW this not compatible with Playnite 9+
	if (($game.Platform.Name) -contains $sNamePCWin)
	{
		$bWalkBackPath = $true
		$bIsConsole = $false
	} elseif (($game.Platform.Name) -contains $sNamePCDOS)
	{
		$bWalkBackPath = $true
		$bIsConsole = $false
	} else {
		$bWalkBackPath = $($($game.platform.name) -in @('Nintendo Wii U','Sony PlayStation 3'))
	}

	if ($bWalkBackPath)
	{
		if (!$bIsConsole)
		{
			[string[]]$asChildFolderNames = @('bin','Bin64','Binaries','client','Content','data','Desktop Version','DOSBOX','en_us','exe','exe64','Final','game','GameData','Game Files','media','pc','release','Release_Steam','resource','runtime','src','StandaloneWindows64','SYSTEM','win','win_x64','Win32','Win32_x86_Release','win32build','Win64','Windows','WindowsNoEditor','x64','linc','engine','Desktop','DeploymentRoot','vctdata')

			$sFolderSubstring = ''
			if ($($game.PlayAction.Path))
			{
#TODO LOW - review - what was this for
				$sFolderSubstring = $($game.PlayAction.Path)
				if ($sFolderSubstring -match '-Win64-Shipping.exe')
				{
					$sFolderSubstring = ($sFolderSubstring.replace('-Win64-Shipping.exe',''))
				} else {
					$sFolderSubstring = ''
				}
			}
		} else {
			# Wii U, PS3, Vita
			[string[]]$asChildFolderNames = @('code','USRDIR','PS3_GAME')
		}

		$bRemovedSubFolderThisLoop = $true
		While ($bRemovedSubFolderThisLoop)
		{

			$bRemovedSubFolderThisLoop = $false

			if (!$sResult)
			{
				$sMsg = 'Should not be possible to reduce the game Installation Directory to an empty string'
				$__logger.Error($sMsg)
				if (!$Silent)
				{
					$null = $PlayniteApi.Dialogs.ShowMessage($sMsg,'IMPOSSIBLE ERROR')
				}
				break
			}

			$sCurrentFolder = Split-Path -Path $sResult -leaf

			if ($sCurrentFolder -in $asChildFolderNames)
			{
				# drop last folder
				$sResult = $sResult.Substring(0, ($sResult.length - $sCurrentFolder.length - 1))

				$bRemovedSubFolderAny = $true
				$bRemovedSubFolderThisLoop = $true
			}

			if ((!$bRemovedSubFolderThisLoop) -and (!$bIsConsole) -and ($sResult))
			{

				# get last folder
				$sCurrentFolder = Split-Path -Path $sResult -leaf

				if (($sFolderSubstring) -and ($sCurrentFolder -eq $sFolderSubstring))
				{
					# drop last folder
					$sResult = $sResult.Substring(0, ($sResult.length - $sCurrentFolder.length - 1))
					$bRemovedSubFolderAny = $true
					$bRemovedSubFolderThisLoop = $true
				} elseif ((($sFriendlyGameName1) -and ($sResult -match $sFriendlyGameName1)) -or (($sFriendlyGameName2) -and ($sResult -match $sFriendlyGameName2))) {
					# last check - test for scenario '*\<gamename>\*\'

					if (($sFriendlyGameName1) -and ($sResult -like "*$sFriendlyGameName1*$sFriendlyGameName1*"))
					{
						# drop last folder
						$sResult = $sResult.Substring(0, ($sResult.length - $sCurrentFolder.length - 1))
						$bRemovedSubFolderAny = $true
						$bRemovedSubFolderThisLoop = $true
					} elseif (($sFriendlyGameName2) -and ($sResult -like "*$sFriendlyGameName2*$sFriendlyGameName2*")) {
						# drop last folder
						$sResult = $sResult.Substring(0, ($sResult.length - $sCurrentFolder.length - 1))
						$bRemovedSubFolderAny = $true
						$bRemovedSubFolderThisLoop = $true
					} elseif (($sFriendlyGameName1) -and ($sCurrentFolder -match $sFriendlyGameName1))
					{
						# fail test
					} elseif (($sFriendlyGameName2) -and ($sCurrentFolder -match $sFriendlyGameName2))
					{
						# fail test
					} else {
						# drop last folder
						$sCurrentFolder = $sResult.Substring(0, ($sResult.length - $sCurrentFolder.length - 1))
						# get last folder (2nd last folder of current result)
						if ($sCurrentFolder)
						{
							$sCurrentFolder = Split-Path -Path $sCurrentFolder -leaf
						}

						# pass test
						if (($sFriendlyGameName1) -and ($sCurrentFolder -match $sFriendlyGameName1))
						{
							# drop last folder
							$sCurrentFolder = Split-Path -Path $sResult -leaf
							$sResult = $sResult.Substring(0, ($sResult.length - $sCurrentFolder.length - 1))
							$bRemovedSubFolderAny = $true
							$bRemovedSubFolderThisLoop = $true
						} elseif (($sFriendlyGameName2) -and ($sCurrentFolder -match $sFriendlyGameName2)) {
							# drop last folder
							$sCurrentFolder = Split-Path -Path $sResult -leaf
							$sResult = $sResult.Substring(0, ($sResult.length - $sCurrentFolder.length - 1))
							$bRemovedSubFolderAny = $true
							$bRemovedSubFolderThisLoop = $true
						}
					}
				}
			}

			if (!$bRemovedSubFolderThisLoop)
			{
				break
			}
		}




		if (!$bRemovedSubFolderAny)
		{
			$sResult = $sGameInstallDir
		}
	}

	# need for Invoke-Item to work
	if ($sResult)
	{
		$sResult = $sResult.TrimEnd('\') + '\'
	}

	# check path exists - 2nd check (should not be needed)
	if ($Verify)
	{
		if (![System.IO.Directory]::Exists($sResult))
		{
			$sMsg = "Game install directory does not exist.`r`n`r`n'$sResult'"
			$__logger.Error($sMsg)
			if (!$Silent)
			{
				$null = $PlayniteApi.Dialogs.ShowMessage($sMsg,'ERROR')
			}
			$sResult = ''
			return $sResult
		}
	}

	return $sResult
}

function Get-EmuLibraryRomFullName([parameter(mandatory)][object]$game)
{
#updated 20211223
	[string]$sResult = ''

	if ($($game.IsInstalled) -eq $true)
	{
		if ($game.GameImagePath)
		{
			$sResult = $PlayniteApi.ExpandGameVariables($game, $game.GameImagePath)
		}
	}

	if (!$sResult)
	{
		$sResult = $($game.GameId)
		if ($sResult -match '\|')
		{
			$sResult = ($sResult -split '\|')[1]
		} else {
			$sResult = ''
		}
	}

	return $sResult
}

function Get-WindowsFriendlyNameLite([parameter()]$value)
{
# remove invalid characters for Windows OS
	$invalidChars = [IO.Path]::GetInvalidFileNameChars() -join ''
	$re = "[{0}]" -f [RegEx]::Escape($invalidChars)
	return ($value -creplace $re)
}

function Get-WindowsFriendlyNameFolder($value)
{
# remove invalid characters for Windows OS
# Windows folder names must not end with space or .
	$invalidChars = [IO.Path]::GetInvalidFileNameChars() -join ''
	$re = "[{0}]" -f [RegEx]::Escape($invalidChars)
	$value = ($value -creplace $re)
	$value = $value.TrimEnd()
	if ($value -match '^\.')
	{
		$value = $value.Substring(1,$value.length - 1)
	}
	if ($value -match '\.$')
	{
		$value = $value.Substring(0,$value.length - 1)
	}
	$value = $value.TrimEnd()
	return ($value)
}