Set-StrictMode -Version Latest
#updated 20230801
#oagIO_SFO.ps1

function Get-TitleIdFromSFO([string]$fullnameSFO)
{
	return (Get-ValueFromSFO -fullnameSFO $fullnameSFO -key 'TITLE_ID')
}

function Get-GameNameFromSFO([string]$fullnameSFO)
{
	return (Get-ValueFromSFO -fullnameSFO $fullnameSFO -key 'TITLE')
}

function Get-ValueFromSFO([string]$fullnameSFO,[string]$key)
{
#updated 20230718
	[string]$sResult = ''
	[string]$sText = ''
	[int]$iSize = 0
	[int]$iOffsetKeyTbl = 0
	[int]$iOffsetDataTbl = 0
	[int]$iTotalIndexTbl = 0
	[int]$i = 0
	[int]$iOffset = 20

	if (![System.IO.File]::Exists($fullnameSFO))
	{
		return
	}

	# $bytes = $null
	# $bytes = [System.IO.File]::ReadAllBytes($fullnameSFO)
	[byte[]]$bytes = Get-Content -LiteralPath $fullnameSFO -Encoding Byte -Raw

	$iSize = $bytes.length

	$iOffsetKeyTbl = $bytes[8] + $(256 * $bytes[9])
	$iOffsetDataTbl = $bytes[12] + $(256 * $bytes[13])
	$iTotalIndexTbl = $bytes[16]	

	# read index table
	[System.Collections.Generic.List[Object]]$colIndexTbl = @()
	For ($i=0; $i -lt $iTotalIndexTbl; $i++)
	{

		[hashtable]$hashEntry = @{}
		$oCustom = $null

		$hashEntry = @{
			offsetKeyTbl = $bytes[$iOffset]
			paramFmt = $bytes[$iOffset + 3]
			paramLen = $bytes[$iOffset + 4] + $($bytes[$iOffset + 5] * 256)
			paramMaxLen = $bytes[$iOffset + 8] + $($bytes[$iOffset + 9] * 256)
			offsetDataTbl = $bytes[$iOffset + 12] + $($bytes[$iOffset + 13] * 256)
		}

		$oCustom = [PsCustomObject]$hashEntry
		$null = $colIndexTbl.Add($oCustom)

		$iOffset = $iOffset + 16
	}

	ForEach ($oCustom in $colIndexTbl)
	{

		if ($oCustom.paramFmt -eq 2)
		{
			$sText = ''
			$sText = Get-TextFromBytes -bytes $bytes -iOffset $($iOffsetKeyTbl + $oCustom.offsetKeyTbl) -iSize 16

			if ($sText -eq $key)
			{

				switch ($sText)
				{
					'TITLE' {
						$sResult = Get-TextFromBytes -bytes $bytes -iOffset $($iOffsetDataTbl + $oCustom.offsetDataTbl) -iSize $oCustom.paramLen

						# a few titles use linefeeds to separate subtitle
						$sResult = $sResult.replace("`r`n",": ").replace("`n",": ")

						break
					}
					'TITLE_ID' {
						$sResult = Get-TextFromBytes -bytes $bytes -iOffset $($iOffsetDataTbl + $oCustom.offsetDataTbl) -iSize $oCustom.paramLen
						break
					}
				}
			}
		}
	}

	return $sResult
}

function Get-TextFromBytes(
	[parameter(mandatory)][Array]$bytes,
	[parameter(mandatory)][int]$iOffset,
	[int]$iSize = 127,
	[bool]$bNullTerminated = $true
) {

	[int]$i = 0
	[int]$iCount = $bytes.Count
	[System.Collections.Generic.List[Byte]]$sBytes = @()

	if ($iOffset -gt $iCount)
	{
		$__logger.Error("offset greater than byte array")
		return
	}

	# fix invalid iSize
	if ($iSize -gt ($iCount - $iOffset))
	{
		$iSize = $($iCount - $iOffset)
	}

	[int]$iStop = $iOffset + $iSize
	For ($i=$iOffset; $i -le $iStop; $i++)
	{

		if ($bNullTerminated -and $bytes[$i] -eq 0)
		{
			break
		} else {
			#$sBytes += $bytes[$i]
			$sBytes.Add($bytes[$i])
		}
	}

	return [System.Text.Encoding]::UTF8.GetString($sBytes)
}