Set-StrictMode -Version Latest
#updated 20230731
#oagIO_XBE.ps1

function Get-RegionIDFromXBE([string]$fullnameXBE) {
	[string]$sResult = ''
	[hashtable]$hashCert = Get-XBECertificate $fullnameXBE
	$sResult = $hashCert.RegionID
	return $sResult
}

function Get-XBECertificate([string]$fullnameXBE) {
#updated 20230731
	[hashtable]$hashResult = @{}
	[string]$sText = ''
	[int]$ptrCERTIFICATE_ADDRESS = 280

	if (![System.IO.File]::Exists($fullnameXBE)) {
		return
	}

	$hashResult = @{
		TitleID = ''
		Title = ''
		RegionID = ''
	}

	[byte[]]$bytes = Get-Content -LiteralPath $fullnameXBE -Encoding Byte -Raw

	[int]$iSize = $bytes.length

	[int]$iOffset = $bytes[$ptrCERTIFICATE_ADDRESS] + $($bytes[$ptrCERTIFICATE_ADDRESS + 1] * 256)

	[int]$iCertOffset = 0x0008
	[int]$iSize = 0x0004

	$sText = Get-EndianHexFromBytes -bytes $bytes -iOffset $($iOffset + $iCertOffset) -iSize $iSize

	$hashResult.TitleID = $sText

	$iCertOffset = 0x000C
	$iSize = 0x0050

	$sText = Get-TextFromBytes -bytes $bytes -iOffset $($iOffset + $iCertOffset) -iSize $iSize -bNullTerminated $false

	$hashResult.Title = $sText

	$iCertOffset = 0x00A0
	$iSize = 0x0004

	$sText = $bytes[$iOffset + $iCertOffset]

	$hashResult.RegionID = $sText

	return $hashResult
}

function Get-EndianHexFromBytes(
	[parameter(mandatory)][Array]$bytes,
	[parameter(mandatory)][int]$iOffset,
	[int]$iSize
) {
	[int]$i = 0
	[int]$iStart = $($iOffset + $iSize) - 1
	[string]$sResult = ''
	[string]$sRaw = 0

	For ($i=$iStart; $i -ge $iOffset; $i--) {

		$sRaw = $bytes[$i]
		$sRaw = [Convert]::ToString($sRaw, 16)

		$sRaw = "{0:X2}" -f $sRaw #NOT WORKING

		# FORCE FIX
		if ($sRaw.length -eq 1) {
			$sRaw = '0' + $sRaw
		}

		$sResult = $sResult + $sRaw
	}
	return $sResult
}

function Get-TextFromBytes(
	[parameter(mandatory)][Array]$bytes,
	[parameter(mandatory)][int]$iOffset,
	[int]$iSize = 127,
	[bool]$bNullTerminated = $true
) {

	[int]$i = 0
	[int]$iCount = $bytes.Count
	[System.Collections.Generic.List[Byte]]$sBytes = @()

	if ($iOffset -gt $iCount) {
		$__logger.Error("offset greater than byte array")
		return
	}

	# fix invalid iSize
	if ($iSize -gt ($iCount - $iOffset)) {
		$iSize = $($iCount - $iOffset)
	}

	[int]$iStop = $iOffset + $iSize
	For ($i=$iOffset; $i -le $iStop; $i++) {

		if ($bNullTerminated -and $bytes[$i] -eq 0) {
			break
		} else {
			#$sBytes += $bytes[$i]
			$sBytes.Add($bytes[$i])
		}
	}

	return [System.Text.Encoding]::UTF8.GetString($sBytes)
}