Set-StrictMode -Version 3

[string]$msMNU_SECTION_THIS = 'Import Data'
[string]$msTHIS_EXT_NAME = 'Import Data from Roms - MetaData'

[int]$iMAX_ROWS_DISPLAY = 45


function Get-IsAdvancedUser
{
	return [bool]$([System.IO.File]::Exists("$CurrentExtensionDataPath\ADVANCED"))
}

function GetGameMenuItems()
{
	param($scriptGameMenuItemActionArgs)

	[bool]$bIsAdvancedUser = Get-IsAdvancedUser
	[string]$sDescriptionTemplate = 'Populate Region field for selected {0} roms (using rom metadata)'
	[string]$sFunctionNameTemplate = "Invoke-StackMain -sPlatform '{0}'"
	[int]$i = 0

	if (!$bIsAdvancedUser)
	{
		[string[]]$sPlatforms = @('Nintendo - Wii U','Sony - PlayStation 3')
	} else {
		[string[]]$sPlatforms = @('Microsoft - XBOX','Nintendo - Wii U','Sony - PlayStation 3','Sony - PlayStation Vita')
		#'Microsoft - XBOX 360'
	}
	[int]$iStop = @($sPlatforms).count

	$arr = New-Object object[] $($iStop + 1)

	For ($i=0; $i -lt $iStop; $i++)
	{
		$arr[$i] = New-Object Playnite.SDK.Plugins.ScriptGameMenuItem
		$arr[$i].Description = $sDescriptionTemplate -f $($("$($sPlatforms[$i])".Split('-')[1]).TrimStart())
		$arr[$i].FunctionName = $sFunctionNameTemplate -f $sPlatforms[$i]
		$arr[$i].Icon = "$CurrentExtensionInstallPath\Icons\{0}.png" -f $sPlatforms[$i]
		$arr[$i].MenuSection = $msMNU_SECTION_THIS
	}

	$arr[$i] = New-Object Playnite.SDK.Plugins.ScriptGameMenuItem
	$arr[$i].MenuSection = $msMNU_SECTION_THIS
	$arr[$i].FunctionName = 'Invoke-ToggleAdvancedOptionsThisExtension'

	if ($bIsAdvancedUser)
	{
		$arr[$i].Description = "Disable the advanced menu options for '$msTHIS_EXT_NAME'"
	} else {
		$arr[$i].Description = "Enable the advanced menu options for '$msTHIS_EXT_NAME'"
	}

	return $arr
}

function Invoke-ToggleAdvancedOptionsThisExtension
{
#updated 20230808
	[bool]$bIsAdvancedUser = Get-IsAdvancedUser
	[string]$fullname = "$CurrentExtensionDataPath\ADVANCED"
	[bool]$bActionSucceeded = $false
	[string]$sMsg = ''

	if (!$bIsAdvancedUser)
	{
		$null = Out-File $fullname -Encoding 'UTF8'
		$bActionSucceeded = [bool]([System.IO.File]::Exists($fullname))
	} else {
		try {
			$null = Remove-Item -LiteralPath $fullname
		} catch {}
		$bActionSucceeded = [bool]$(![System.IO.File]::Exists($fullname))
	}

	# display msg
	if ($bIsAdvancedUser)
	{
		$sMsg = 'disable'
	} else {
		$sMsg = 'enable'
	}
	if ($bActionSucceeded)
	{
		$sMsg = "Advanced options have been " + $sMsg + "d.`r`n`r`nPress F12 after closing this message to refresh the menus"
	} else {
		$sMsg = "Failed to " + $sMsg + " advanced options for this extension."
	}

	$null = $PlayniteApi.Dialogs.ShowMessage($sMsg)
}

function Invoke-StackMain([string]$sPlatform,[switch]$Force)
{


	# load functions
	. "$CurrentExtensionInstallPath\lib\oagGamePath.ps1"
	. "$CurrentExtensionInstallPath\lib\oagEmulationRomCodes.ps1"
	. "$CurrentExtensionInstallPath\lib\oagGameFields.ps1"
	

	[int]$iCountMatched = 0
	[int]$iCountUpdated = 0
	[int]$iCountMissingGame = 0
	[int]$iCountMissingFile = 0
	[int]$iCountFailOther = 0
	[int]$iCountExisting = 0
	[bool]$bAskUser = $Force
	[int]$iCountUpdated = 0
	[string]$sExportPath = 0
	[string]$fullnameRomExe = ''
	[string]$sMsg = ''
	[string]$sNewValue = ''
	[string]$sRegionGuid = ''
	[string]$sSrcFileName = ''
	[string]$fullnameSrc = ''
	[string]$sSrcDesc = 'metadata'
	$sbListNewLined = [System.Text.StringBuilder]''

	# read config for this platform
	[string]$fullnameConfig = "$CurrentExtensionInstallPath\Resources\$sPlatform.json"
	if (![System.IO.File]::Exists($fullnameConfig))
	{
		$__logger.Error("no config file at '$fullnameConfig'")
		exit
	}

	$JsonConfig = [System.IO.File]::ReadAllLines($fullnameConfig) | ConvertFrom-Json

	[string]$sPlatformFull = $JsonConfig.PlatformFull
	[string]$sPlatformShort = $JsonConfig.PlatformShort
	[string]$sPathRemove = $JsonConfig.PathRemove

	$sSrcFileName = $JsonConfig.FileNameMetadata

	if (!$sSrcFileName)
	{
		$__logger.Error("no filename set in config file")
		exit
	}
	if (!$sPlatformFull)
	{
		$__logger.Error("no platform set in config file")
		exit
	}


	# load functions
	switch ([System.IO.Path]::GetExtension($sSrcFileName))
	{
		'.sfo' {
			. "$CurrentExtensionInstallPath\lib\oagIO_SFO.ps1"
		}
		'.xml' {
			. "$CurrentExtensionInstallPath\lib\oagIO_XML.ps1"
		}
		'.xbe' {
			. "$CurrentExtensionInstallPath\lib\oagIO_XBE.ps1"
		}
		'.xex' {
#TODO nicetohave XEX
			exit
		}	
	}



	# get games
	try {
		$selectedGames = $PlayniteApi.MainView.SelectedGames | Where-Object { (($_.Platform.name -eq $sPlatformFull) -and ($_.GameImagePath)) } | Sort-Object {$_.name}

		if (!$Force)
		{
			$selectedGames = $($selectedGames | Where-Object { (-not ($_.Region)) })
		}
	} catch {}

	[int]$iTotalGamesProcessed = @($selectedGames).count

	if ($iTotalGamesProcessed -eq 0)
	{
		$sMsg = 'no games in selection met requirements'
		$null = $PlayniteApi.Dialogs.ShowMessage($sMsg,'ERROR')
		exit
	}

	# Create collection for processed games
	[System.Collections.Generic.List[Object]]$colGamesProcessed = @()


	foreach ($game in $selectedGames)
	{

		# reset
		$sNewValue = ''
		$sMsg = ''
		$fullnameRomExe = ''
		$fullnameSrc = ''


		# get game path
		if ($game.GameImagePath)
		{
			$fullnameRomExe = $PlayniteApi.ExpandGameVariables($game, $game.GameImagePath)
		} elseif ($game.GameId -match '\|') {
			# probably EmuLibrary and uninstalled
			$fullnameRomExe = Get-EmuLibraryRomFullName -game $game
		}

		# verify game path
		if (!([System.IO.File]::Exists($fullnameRomExe)))
		{

			# check if rom on disconnected drive
			if ($fullnameRomExe.length -gt 4)
			{
				if (![System.IO.Directory]::Exists($($fullnameRomExe[0]+$fullnameRomExe[1])))
				{
					$sMsg = "skip game - drive '$($fullnameRomExe[0])' not connected `"$($game.name)`""
				}
			}

			if (!$sMsg)
			{
				if ($game.IsInstalled -eq $true)
				{
					$sMsg = "skip game - invalid path '$fullnameRomExe' for game `"$($game.name)`""
				} else {
					$sMsg = "skip game - game not installed `"$($game.name)`""
				}
			}

			$null = $sbListNewLined.AppendLine($sMsg)
			$iCountMissingGame++
			continue
		}

		# verify game format
		if ([System.IO.Path]::GetExtension($fullnameRomExe) -notin @('.xbe','.xex','.rpx','.bin'))
		{
			$sMsg = "skip game - rom is not in a compatible format `"$($game.name)`""
			$null = $sbListNewLined.AppendLine($sMsg)
			$iCountFailOther++
			continue
		}

		# get path of src file
		$fullnameSrc = Split-Path -Path $fullnameRomExe -parent
		if ($sPathRemove)
		{
			$fullnameSrc = $fullnameSrc.replace("\$sPathRemove",'\')
		}
		$fullnameSrc = Join-Path -Path $fullnameSrc -ChildPath $sSrcFileName

		# verify file exists
		if (![System.IO.File]::Exists($fullnameSrc))
		{
			$iCountMissingFile++
			$sMsg = "skip game - no matching $sSrcDesc file located in the rom folder for game `"$($game.name)`" (expected $sSrcDesc path '$fullnameSrc')"
			$null = $sbListNewLined.AppendLine($sMsg)
			continue
		}

		# process file
		[string]$sTitleID = ''
		switch ([System.IO.Path]::GetExtension($fullnameSrc))
		{
			'.sfo' {
				$sTitleID = Get-TitleIdFromSFO -fullnameSFO $fullnameSrc
				$sNewValue = Get-RegionFromTitleIdSony -TitleId $sTitleID
			}
			'.xml' {
				$contentExtSrc = $null
				$contentExtSrc = Get-Content -LiteralPath $fullnameSrc
				$sTitleID = Get-XmlElementsTextValue -XmlDocument $contentExtSrc -ElementPath 'menu.product_code'
				$sNewValue = Get-RegionFromNintendoGameID -GameID $sTitleID
			}
			'.xbe' {
				$sTitleID = Get-RegionIDFromXBE $fullnameSrc # actually region ID - recycling variable
				$sNewValue = Get-RegionFromXboxRegionID $sTitleID
			}
			'.xex' {
#TODO nicetohave - XEX
			}
		}

		if (!$sNewValue)
		{
			$sMsg = "skip game - could not read region `"$($game.name)`""
			$null = $sbListNewLined.AppendLine($sMsg)
			continue
		}


		[string]$sGuidRegion = ''

		$sGuidRegion = Get-DatabaseFieldGUID -FieldName 'Regions' -value $sNewValue -Force

		if ("$($game.RegionId)".length -gt 0)
		{

			# check does not have same value
			if ($($game.RegionId) -eq $sGuidRegion)
			{
				$sMsg = "skip game - game already has this region recorded `"$($game.name)`""
				$null = $sbListNewLined.AppendLine($sMsg)
				continue
			}

			# request user permission to replace existing value
			if ($bAskUser)
			{
				$sMsg = "Detected Region '$sNewValue'"
				$sMsg = $sMsg + "`r`n`r`nDo you wish to replace the existing value for the game `"$($game.name)`" ($($game.Platform.name)) ?"
				$UserResponse = $PlayniteApi.Dialogs.ShowMessage($sMsg, $msTHIS_EXT_NAME, 4)
				if ($UserResponse -ne 'Yes')
				{
					$sMsg = "skip game - user did not want to replace the existing value `"$($game.name)`""
					$null = $sbListNewLined.AppendLine($sMsg)
					continue
				}
			}
		}

		# update game
		$game.RegionId = [guid]$sGuidRegion
		$game.Modified = (Get-Date)
		$null = $PlayniteApi.Database.Games.Update($game)
		$sMsg = "Updated game. Set Region to '$sNewValue' for `"$($game.name)`" ($($game.Platform.name))"
		$__logger.Info($sMsg)

		$iCountUpdated++
		$null = $colGamesProcessed.Add($game)
	}


	# Show finish dialogue with results and ask if user wants to export results
	$sMsg = ''
	if ($sbListNewLined)
	{
		$sMsg = $($sbListNewLined.ToString())
	}

	$sMsg = "`r`n`r`n$sMsg"

	if (@($sbListNewLined).count -gt $iMAX_ROWS_DISPLAY)
	{	
		$__logger.Error($sMsg)
		$sMsg = ''
		$sMsg = "`r`nErrors occurred. See Playnite log for details."
	}


	if ($iCountUpdated -gt 0)
	{
		$sMsg = "Added region to $iCountUpdated games.$sMsg`r`n`r`nDo you want to export these results?"
	} else {
		$sMsg = "No regions were populated.$sMsg"
	}

	if ($iCountUpdated -gt 0)
	{
		$UserResponse = $PlayniteApi.Dialogs.ShowMessage($sMsg, $msTHIS_EXT_NAME, 4)
		if ($UserResponse -eq "Yes")
		{
			$sExportPath = $PlayniteApi.Dialogs.SaveFile("CSV|*.csv|Formatted TXT|*.txt")
			if ($sExportPath)
			{
				if ($sExportPath -match ".csv$")
				{
					$colGamesProcessed | Select-Object Name, GameImagePath, Region, Platform | ConvertTo-Csv -NoTypeInformation | Out-File $sExportPath -Encoding 'UTF8'
				} else {
					$colGamesProcessed | Select-Object Name, GameImagePath, Region, Platform | Format-Table -AutoSize | Out-File $sExportPath -Encoding 'UTF8'
				}
				$null = $PlayniteApi.Dialogs.ShowMessage('Results exported successfully.', $msTHIS_EXT_NAME)
			}
		}
	} else {
		$null = $PlayniteApi.Dialogs.ShowMessage($sMsg)
	}
}